# Copyright 2024 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, http
from odoo.exceptions import ValidationError
from odoo.http import request
from odoo.tools import str2bool

from odoo.addons.website_event.controllers.main import WebsiteEventController
from odoo.addons.website_event_questions.controllers.main import WebsiteEvent


class WebsiteEventPartnerQuestion(WebsiteEvent):
    def _process_attendees_form(self, event, form_details):
        """Process data posted from the attendee details form."""

        registrations = super()._process_attendees_form(event, form_details)
        fields_to_update = {}
        for key, value in form_details.items():
            # questions contact
            if key.startswith("question_contact-"):
                _dummy, question_str = key.split("-")
                # ruff: noqa: B023
                question_id = event.sudo().partner_question_ids.filtered(
                    lambda q: q.id == int(question_str)
                )
                if not question_id:
                    raise ValidationError(_("Answer not in event."))

                ir_field_id = question_id.field_id.ir_field_id
                if ir_field_id.ttype == "integer":
                    fields_to_update[ir_field_id.name] = int(value) if value else False
                if ir_field_id.ttype == "many2one":
                    fields_to_update[ir_field_id.name] = int(value) if value else False
                elif ir_field_id.ttype == "float":
                    fields_to_update[ir_field_id.name] = (
                        float(value) if value else False
                    )
                elif ir_field_id.ttype == "boolean":
                    fields_to_update[ir_field_id.name] = (
                        str2bool(value) if value else False
                    )
                else:
                    fields_to_update[ir_field_id.name] = value

            # multi question
            if key.startswith("question_contact_multi-"):
                _dummy, question, _answer = key.split("-")
                # ruff: noqa: B023
                question_id = event.sudo().partner_question_ids.filtered(
                    lambda q: q.id == int(question)
                )
                if not question_id:
                    raise ValidationError(_("Answer not in event."))
                field_name = question_id.field_id.ir_field_id.name
                if fields_to_update.get(field_name):
                    fields_to_update[field_name][0][2].append(int(value))
                else:
                    fields_to_update[field_name] = [(6, 0, [int(value)])]

            # when seeing lastname, search firstname
            # to complete name of registered participant
            if key.endswith("-lastname"):
                lastname = value
                counter_str, _lastname = key.split("-")
                counter = int(counter_str) - 1
                firstname = form_details.get(counter_str + "-firstname", "")
                name = request.env["res.partner"]._get_computed_name(
                    lastname, firstname
                )
                registrations[counter]["name"] = name

        request.env.user.partner_id.update(fields_to_update)

        return registrations


class WebsiteEventControllerSolagro(WebsiteEventController):
    # overload route to add more precompletion info on default_first_attendee
    # copy-paste by lack of extensibility
    # everything is the same except default_first_attendee firstname and lastname
    # fields
    @http.route(
        ['/event/<model("event.event"):event>/registration/new'],
        type="json",
        auth="public",
        methods=["POST"],
        website=True,
    )
    def registration_new(self, event, **post):
        tickets = self._process_tickets_form(event, post)
        availability_check = True
        if event.seats_limited:
            ordered_seats = 0
            for ticket in tickets:
                ordered_seats += ticket["quantity"]
            if event.seats_available < ordered_seats:
                availability_check = False
        if not tickets:
            return False
        default_first_attendee = {}
        if not request.env.user._is_public():
            default_first_attendee = {
                "name": request.env.user.name,
                "firstname": request.env.user.firstname,
                "lastname": request.env.user.lastname,
                "email": request.env.user.email,
                "phone": request.env.user.mobile or request.env.user.phone,
            }
        else:
            visitor = request.env["website.visitor"]._get_visitor_from_request()
            if visitor.email:
                default_first_attendee = {
                    "name": visitor.name,
                    "firstname": visitor.partner_id.firstname,
                    "lastname": visitor.partner_id.lastname,
                    "email": visitor.email,
                    "phone": visitor.mobile,
                }
        return request.env["ir.ui.view"]._render_template(
            "website_event.registration_attendee_details",
            {
                "tickets": tickets,
                "event": event,
                "availability_check": availability_check,
                "default_first_attendee": default_first_attendee,
            },
        )
